/*
 * $RCSfile: scout_thread_arch.h,v $
 *
 * x-kernel v3.3
 *
 * Copyright (c) 1993,1991,1990,1996  Arizona Board of Regents
 *
 * $Log: scout_thread_arch.h,v $
 * Revision 1.1  1997/01/18 00:33:45  dlarson
 * Initial revision
 *
 *
 */
/*
 * This must not be included by anything but thread.c.  That's
 * why this file is not protected by #ifdefs.
 */

/*
 * x86 register usage (gcc):
 *
 * Callee saved registers:	ebx, edi, esi, ebp
 *	ebp = frame pointer (FP)
 *	esp = stack pointer (SP)
 *
 * Callee does not need to save any floating point regs, so the FPU macros
 * are not needed.
 *
 */

#define MAGIC_COOKIE		0xFEEDF001
#define MIN_STACK_SIZE		(1024)	/* enough for a single printf() */
#define MIN_USEFUL_STACK_SIZE	(2*MIN_STACK_SIZE)
#define DEFAULT_STACK_SIZE	(8*MIN_USEFUL_STACK_SIZE)
#define FPU_STATE_SIZE		(9*sizeof(long))


/*
 * Switch to stack TO_SP, discarding the old stack pointer and invoke
 * start().
 */
#define SWITCH_STACK_AND_START(to_sp, start)				\
{									\
    asm volatile							\
      ("movl %0, %%esp
	jmp  start
       ":						/* output */	\
	: "r" (to_sp), "i" (start));			/* input */	\
}


/*
 * Save necessary state on stack, save new stack pointer to
 * variable saved_sp.
 */
#define SAVE_STATE(saved_sp)						\
{									\
    asm volatile							\
      ("push %%ebx
	push %%edi
	push %%esi
	push %%ebp
	movl %%esp, %0
       ": "m=" (saved_sp)				/* output */	\
	:						/* input */	\
	: "memory");					/* modified */	\
}


/*
 * Load state stored at FROM_SP.
 */
#define LOAD_STATE(from_sp)						\
{									\
    asm volatile							\
      ("movl %0, %%esp
	popl %%ebp
	popl %%esi
	popl %%edi
	popl %%ebx
       ":						/* output */	\
	: "m" (from_sp));				/* input */	\
}

#define SAVE_FPU_STATE(fpu_state)
#define LOAD_FPU_STATE(fpu_state)

#define FPU_ON
#define FPU_OFF

/* this is for debugging purposes only: */
#define GET_SAVED_PC(sp)

